export interface OnigCaptureIndex {
    index: number;
    start: number;
    end: number;
    length: number;
}
export interface OnigMatch {
    index: number;
    captureIndices: OnigCaptureIndex[];
    scanner: OnigScanner;
}
declare class OnigScanner {
    private sources;
    /**
     * Create a new scanner with the given patterns
     * @param patterns  An array of string patterns
     */
    constructor(patterns: string[]);
    readonly patterns: string[];
    /**
     * Find the next match from a given position
     * @param string The string to search
     * @param startPosition The optional position to start at, defaults to 0
     * @param callback The (error, match) function to call when done, match will null when there is no match
     */
    findNextMatch(string: string, startPosition: number, callback: (err, match?: OnigMatch) => void): void;
    /**
     * Find the next match from a given position
     * @param string The string to search
     * @param startPosition The optional position to start at, defaults to 0
     */
    findNextMatchSync(string: string, startPosition: number): OnigMatch;
    convertToString(value: any): any;
    convertToNumber(value: any): any;
}
export default OnigScanner;
